<?php

namespace Drupal\aws_cloud\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Link;

/**
 * Plugin implementation of the 'ip_permission_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "ip_permission_formatter",
 *   label = @Translation("IpPermission formatter"),
 *   field_types = {
 *     "ip_permission"
 *   }
 * )
 */
class IpPermissionFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $rows = [];

    $security_group = $this->getSecurityGroupEntity();
    foreach ($items as $delta => $item) {
      /* @var \Drupal\aws_cloud\Plugin\Field\FieldType\IpPermission $item */
      if (!$item->isEmpty()) {
        $revoke_link = Link::createFromRoute($this->t('Revoke'), 'entity.aws_cloud_security_group.revoke_form', [
          'cloud_context' => $security_group->getCloudContext(),
          'aws_cloud_security_group' => $security_group->id(),
          'type' => $items->getName(),
          'position' => $delta,
        ]);
        $rows[] = [
          ($item->ip_protocol == "-1") ? $this->t('All Traffic') : $item->ip_protocol,
          $item->from_port,
          $item->to_port,
          $item->cidr_ip,
          $item->cidr_ip_v6,
          $item->prefix_list_id,
          $item->group_id,
          $item->group_name,
          $item->peering_status,
          $item->user_id,
          $item->vpc_id,
          $item->peering_connection_id,
          $revoke_link,
        ];
      }
    }

    if (count($rows)) {
      $elements[0] = [
        '#theme' => 'table',
        '#header' => [
          $this->t('IP Protocol'),
          $this->t('From Port'),
          $this->t('To Port'),
          $this->t('CIDR IP'),
          $this->t('CIDR IP V6'),
          $this->t('Prefix List Id'),
          $this->t('Group Id'),
          $this->t('Group Name'),
          $this->t('Peering Status'),
          $this->t('Group User Id'),
          $this->t('VPC ID'),
          $this->t('Peering Connection Id'),
          $this->t('Operation'),
        ],
        '#rows' => $rows,
      ];
    }

    return $elements;
  }

  /**
   * Helper function to retrieve the Security Group from the route.
   */
  private function getSecurityGroupEntity() {
    $security_group = FALSE;
    foreach (\Drupal::routeMatch()->getParameters() as $param) {
      if ($param instanceof \Drupal\Core\Entity\EntityInterface) {
        $security_group = $param;
      }
    }
    return $security_group;
  }

}
