<?php

namespace Drupal\cloud\Service;

use Drupal\Core\Url;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Utility\LinkGenerator;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Entity link element renderer service.
 */
class EntityLinkRenderer implements EntityLinkRendererInterface {

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  private $routeMatch;

  /**
   * The link generator service.
   *
   * @var \Drupal\Core\Utility\LinkGenerator
   */
  private $linkGenerator;

  /**
   * An entity type manager instance.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * Constructs a new EntityLinkRenderer object.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The current route match.
   * @param \Drupal\Core\Utility\LinkGenerator $link_generator
   *   The link generator service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   An entity type manager instance.
   */
  public function __construct(
    RouteMatchInterface $route_match,
    LinkGenerator $link_generator,
    EntityTypeManagerInterface $entity_type_manager) {

    $this->routeMatch = $route_match;
    $this->linkGenerator = $link_generator;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function renderViewElement(
    $value,
    $target_type,
    $field_name,
    array $query = [],
    $alt_text = '') {

    $cloud_context = $this->routeMatch->getParameter('cloud_context');

    if (is_array($value)) {
      $values = $value;
    }
    else {
      $values = [$value];
    }

    $htmls = [];
    foreach ($values as $value) {
      $entity_ids = $this->entityTypeManager
        ->getStorage($target_type)
        ->getQuery()
        ->condition($field_name, $value)
        ->execute();

      if (empty($entity_ids)) {
        $htmls[] = $value;
      }
      else {
        $htmls[] = $this->linkGenerator->generate(
          !empty($alt_text) ? $alt_text : $value,
          Url::fromRoute(
            "entity.$target_type.canonical",
            [
              'cloud_context' => $cloud_context,
              $target_type => array_values($entity_ids)[0],
            ],
            ['query' => $query]
          )
        );
      }
    }

    return [
      '#markup' => implode(', ', $htmls),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function renderFormElements(
    $value,
    $target_type,
    $field_name,
    array $options,
    $alt_text = '') {

    return $this->renderViewElement($value, $target_type, $field_name, $options, $alt_text)
      + $options
      + ['#type' => 'item'];
  }

}
